/*
** lzcommon.h - Shared information for LZ modules.
**
** Author:  DavidDi
*/


// Constants
/////////////

#define RING_BUF_LEN       4096        // size of ring buffer
#define MAX_RING_BUF_LEN   4224        // size of ring buffer - from LZFile
                                       // struct declaration in lzexpand.h

#define NIL                RING_BUF_LEN   // flag index used in binary search
                                          // trees

#define BUF_CLEAR_BYTE     ((uint8_t) ' ')   // rgbyteRingBuf[] initializer

#define MAX_LITERAL_LEN    2           // encode string into position and
                                       // length if match length greater than
                                       // this value (== # of bytes required
                                       // to encode position and length)

#define FIRST_MAX_MATCH_LEN   0x10     // ALG_FIRST used this length
#define LZ_MAX_MATCH_LEN      (0x10 + MAX_LITERAL_LEN)
#define LZA_MAX_MATCH_LEN     64
                                       // upper limit for match length
                                       // (n.b., assume length field implies
                                       // length += 3)

// Maximum number of bytes LZDecode() and LZADecode() will expand beyond
// position requested.
#define MAX_OVERRUN        ((long)pLZI->cbMaxMatchLen)


// Globals
///////////

extern int iCurMatch,      // index of longest match (set by LZInsertNode())
           cbCurMatch;     // length of longest match (set by LZInsertNode())

extern uint32_t uFlags;    // LZ decoding description byte

extern int iCurRingBufPos; // ring buffer offset

// Prototypes
//////////////

// lzcommon.c
extern unsigned int LZInitTree(LZINFO *pLZI);
extern void LZFreeTree(LZINFO *pLZI);
extern void LZInsertNode(int nodeToInsert, unsigned int bDoArithmeticInsert, LZINFO *pLZI);
extern void LZDeleteNode(int nodeToDelete, LZINFO *pLZI);

// lzcomp.c
extern int LZEncode(int doshDest, const char *doshSource, LZINFO *pLZI);

// lzexp.c
extern int LZDecode(int doshDest, const char *doshSource, uint32_t cblExpandedLength,
                    /*BOOL bRestartDecoding,*/ unsigned int bFirstAlg, LZINFO *pLZI);

